# -----------------------------------------------------------------------------
# The type of a spectrum (eg hnco, hcch-tocsy, ...) can be used together
# with the molecule primary structure to infer expected peaks.
#
import string

import pyutil
import sparky
import sputil
import tkutil

# -----------------------------------------------------------------------------
# Name, dimension, aliases
#
spectrum_types = (
  ('n15hsqc', 2, ()),
  ('hnco', 3, ()),
  ('hnca', 3, ()),
  ('hncoca', 3, ('caconh',)),
  ('hncacb', 3, ()),
  ('cbcaconh', 3, ('hncocacb',)),
  ('hnha', 3, ()),
  ('haconh', 3, ('hncoha',)),
  )
spectrum_type_names = map(lambda nda: nda[0], spectrum_types)

# -----------------------------------------------------------------------------
#
def spectrum_type(spectrum):

  #
  # The spectrum type is saved using the (key,value) data extension mechanism.
  #
  if not hasattr(spectrum, 'type'):
    spectrum.type = spectrum.saved_value('spectrum_type')
    
  if spectrum.type == None:
    spectrum.type = guess_spectrum_type(spectrum)
    if spectrum.type:
      spectrum.save_value('spectrum_type', spectrum.type)
      
  if spectrum.type == None:
    d = tkutil.the_dialog(spectrum_type_dialog)
    spectrum.type = d.get_type(spectrum.name)
    if spectrum.type:
      spectrum.save_value('spectrum_type', spectrum.type)

  return spectrum.type

# -----------------------------------------------------------------------------
#
def change_spectrum_type():

  spectrum = sputil.selected_spectrum()
  if spectrum:
    d = tkutil.the_dialog(spectrum_type_dialog)
    spectrum.type = d.get_type(spectrum.name)
    if spectrum.type:
      spectrum.save_value('spectrum_type', spectrum.type)

# -----------------------------------------------------------------------------
#
def guess_spectrum_type(spectrum):

  type_name = None
  longest_match = 0
  for name, dim, aliases in spectrum_types:
    if dim == spectrum.dimension:
      match_length = 0
      for t in (name,) + aliases:
        if len(t) > match_length and string.find(spectrum.name, t) >= 0:
          match_length = len(t)
      if match_length > longest_match:
        type_name = name
        longest_match = match_length
  
  return type_name

# -----------------------------------------------------------------------------
#
class spectrum_type_dialog(tkutil.Dialog):

  def __init__(self):

    tkutil.Dialog.__init__(self, 'Spectrum Type')

    tm = tkutil.option_menu(self.top, 'Spectrum type: ', spectrum_type_names)
    tm.frame.pack(side = 'top')
    self.type_menu = tm

    br = tkutil.button_row(self.top,
			   ('Ok', self.ok_cb),
                           ('Cancel', self.close_cb),
                           ('Help', sputil.help_cb('SpectrumType')),
			   )
    br.frame.pack(side = 'top', anchor = 'w')

  # ---------------------------------------------------------------------------
  #
  def ok_cb(self):

    self.type = self.type_menu.get()
    self.close_cb()

  # ---------------------------------------------------------------------------
  #
  def get_type(self, spectrum_name):

    self.type = None

    self.type_menu.label['text'] = spectrum_name + ' spectrum type: '
    initial = pyutil.closest_string(spectrum_name, spectrum_type_names)
    self.type_menu.set(initial)
    
    self.top.title(spectrum_name + ' spectrum type')
    self.show_window(1)
    
    self.return_when_closed()

    return self.type
